#pragma once

#include <visa.h>
#include <string>
#include <map>

// Define the map of amplifier models
enum CommandSetType {CS_LONG_RF_MAN, CS_LONG_RF_CW, CS_LONG_OP_MAN, CS_LONG_OP_CW, CS_4LINE, CS_SHORT, CS_2LINE};
extern std::map<std::string, CommandSetType> AmpModelMap;

class Amplifier
{
public:

	enum ModeType {MANUAL, PULSE, ALC_INTERNAL, ALC_EXTERNAL};
	enum BandType {LOW, HIGH,AUTO};
	enum ParityType {NONE,ODD,EVEN,MARK,SPACE};
	enum FlowControlType {NO_FLOW_CONTROL, XON_XOFF,RTS_CTS,DTR_DSR};
	enum StopBitsType {ONE, ONE5,TWO};
	enum VSWRLimit {VSWR_LIMIT_NONE, VSWR_LIMIT_6TO1, VSWR_LIMIT_3TO1, VSWR_LIMIT_2TO1};

	Amplifier();
	Amplifier(std::string Model);
	void SetModel(std::string Model);
	int Initialize(std::string VisaResourceName,
					bool reset = true, 
					int BaudRate = 9600,
					FlowControlType FlowControl = NO_FLOW_CONTROL, 
					ParityType Parity = NONE,
					int DataBits = 8,
					StopBitsType StopBits = ONE);
	int Reset();
	int Power(bool State);
	int RF_Output(bool State);
	unsigned int ReadFault();
	unsigned int ReadFaultLogCount();
	int ReadFaultLog(unsigned int faultNumber, unsigned int *faultCode, int *PowerOnHours, int *RF_OnHours);
	int ClearFaultLog();
	unsigned int ReadState();

	int ConfigureRFGain(unsigned int Gain);	
	int ConfigureMode(ModeType Mode);
	int ConfigureBand(BandType BandSetting);
	int Configure_ALC_DetectorGain(unsigned int Gain);
	int Configure_ALC_Response(unsigned int response);
	int Configure_ALC_Threshold(unsigned int threshold);
	int ConfigureVSWRLimit(VSWRLimit limit);
	int SetFrequency(double frequency);

	unsigned int QueryRFGain();
	unsigned int Query_ALC_DetectorGain();	
	unsigned int Query_ALC_Response();
	unsigned int Query_ALC_Threshold();
	float QueryForwardPower();
	float QueryReversePower();
	int QueryOperatingHours(int *PowerOnHours, int *RF_OnHours);
	VSWRLimit QueryVSWRLimit();
	double QueryFrequency();
	std::string QueryID();
	int Close();

	void GetValidModels(std::string ModelNames[], int *count);


private:

	ViSession VISAresource;
	std::string AmpModel;
	CommandSetType CommandSet;

	int ConfigureSerial(int BaudRate, 
						FlowControlType FlowControl,
						ParityType Parity, 
						int DataBits, 
						StopBitsType StopBits);

	int SendCommand(char *command,  char *Response = NULL, bool Query = false, unsigned int waitAfterSend = 20,  unsigned int BytesToRead = 100);

	std::string popToken(std::string& text, const std::string& delimiter);
};
